window.onload = () => {
  let spread = new GC.Spread.Sheets.Workbook(document.getElementById("ss"));
  registerCommand(spread);
};

const subscriptions = {};
const getNextUniqueId = getIdGenerator();

function getIdGenerator() {
  let lastId = 0;

  return function getNextUniqueId() {
    lastId += 1;
    return lastId;
  };
}

function subscribe(eventType, callback) {
  const id = getNextUniqueId();

  if (!subscriptions[eventType]) subscriptions[eventType] = {};

  subscriptions[eventType][id] = callback;

  return {
    unsubscribe: () => {
      delete subscriptions[eventType][id];
      if (Object.keys(subscriptions[eventType]).length === 0)
        delete subscriptions[eventType];
    }
  };
}

function publish(eventType, arg) {
  if (!subscriptions[eventType]) return;
  Object.keys(subscriptions[eventType]).forEach(key =>
    subscriptions[eventType][key](arg)
  );
}

const subscription = subscribe("spanAdded", arg =>
  console.log("SpanAdded:", arg)
);

function addSpan() {
  let spread = GC.Spread.Sheets.findControl(document.getElementById("ss"));
  let sheet = spread.getActiveSheet();
  spread.commandManager().execute({
    cmd:"addSpan",
    sheetName: sheet.name(),
    spread: spread
})
}
function registerCommand(spread) {
  let addSpanCmd = {
    canUndo: true,
    execute: (spread, options, isUndo) => {
      let Commands = GC.Spread.Sheets.Commands;
      if (isUndo) {
        Commands.undoTransaction(spread, options);
        return true;
      } else {
        Commands.startTransaction(spread, options);
        spread.suspendPaint();
        let sheet = spread.getActiveSheet();

        let sel = sheet.getSelections()[0];
        sheet.addSpan(sel.row, sel.col, sel.rowCount, sel.colCount);

        publish("spanAdded", {
          spread: spread,
          sheet: sheet,
          row: sel.row,
          col: sel.col,
          rowCount: sel.rowCount,
          colCount: sel.colCount
        });
        spread.resumePaint()
        Commands.endTransaction(spread, options);
        return true;
      }
    }
  };
  let cm = spread.commandManager();
  cm.register("addSpan", addSpanCmd);
}
